//+------------------------------------------------------------------+
//|                                               LuKu_EDiff_v12.mq4 |
//|                                                             LuKu |
//|                                               http://luku.cz/fx/ |
//+------------------------------------------------------------------+
/*
    Indicator displays differential curve by various method
    Inspired by http://www.forexfactory.com/showthread.php?t=160912&page=233
    
    Version 1.0 (12.1.2012)
      - core programming
      - several types of measurement deviations
      - added alert system
      - customizable value of alert lines
      - alert lines calculation by quantile or value
      - added simulated profit calculation on close bars
      - added automated calculation of correlation and direction trades
      
    Version 1.1 (16.1.2012)
      - trade support
      - added close limit for value a quantile
      - time limit for trading
      - show price label
      - spread filtering for trading
      - modify view
      - support AlertQuantileClose to zero
      
    Version 1.2 (18.1.2012)
      - support MinTickCount on alert lines (sometime values jump over alert limit and back bellow)
      - support Stochastic mode
      - bugfix in time filtration
      
    TODO:
      - show shadows tick curve and save it (ShowShadows = true;)
      - export simulated trade and analysis MAE MFE (QAE QFE + forward analysis with those settings)
      - re-calculateSimulatedProfit afret X bars, no only first run
      
    Tradable pairs (positive simulated profit):
      EURUSD / GBPUSD M5
      EURJPY / USDCHF M5
      AUDUSD / USDCAD M5
      NZDUSD / USDCAD M5
      
*/
#property copyright     "LuKu"
#property link          "http://luku.cz/fx/"
string Indicator_Name = "LuKu_EDIFF_v12";
string ShortName      = "EDI";

#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color1 Red
#property indicator_color2 Green
#property indicator_color3 DodgerBlue
#property indicator_color4 White
#property indicator_color5 OrangeRed
#property indicator_style1 STYLE_DOT
#property indicator_style2 STYLE_DOT
#property indicator_style3 STYLE_SOLID
#property indicator_style4 STYLE_SOLID
#property indicator_style5 STYLE_SOLID
#property indicator_width1 1
#property indicator_width2 1
#property indicator_width3 1
#property indicator_width4 1
#property indicator_width5 1

// parameters
extern string    sDiffType             = "------- 1=StdDev 2=MAPips 3=Stoch";
extern int       DiffType              = 1;
extern string    DiffPair              = "GBPUSD";
extern bool      ForceInvert           = false;
extern bool      CorrelationDirection  = true;
extern int       CorelationD1Period    = 50;

extern string    sStdDev          = "------- 1 StdDevMA settings";
extern int       StdDevPeriod     = 20;

extern string    sMAPips          = "------- 2 MAPips settings";
extern int       MaPipsPeriod     = 100;

extern string    sStoch           = "------- 3 Stochastic settings";
extern int       Kperiod          = 14;
extern int       Dperiod          = 3;
extern int       Slowing          = 3;
extern int       SmothCurveFactor = 0;
extern bool      NoSmothActualBar = true;

extern string    sAlertSystem        = "-------- Alert [Value|Quantile]";
extern string    AlertSystem         = "Quantile";
extern bool      SendAlert           = true;
extern bool      SendEmail           = false;
extern string    AlertQuantileArray  = "80,92,98";
extern double    AlertQuantileClose  = 10;
extern int       AlertQRecalcBars    = 10; // recalculate quantile values every N bars
extern string    AlertValueArray     = "2.1,2.7,3.4";
extern double    AlertValueClose     = 0.3;
extern int       AlertMinTickCount   = 2; // fast jump over alert lines protection

extern string    sOther                = "------- Others settings";
extern bool      ShowOnlyDiffCurve     = true;
extern int       MaxBarCount           = 10000; // num of calculated bars 0 = all
extern bool      ShowAlertLevelStatus  = true;
extern bool      ShowMagickNumber      = true;
extern bool      ShowPriceLabel        = true;

extern string    sProfit   = "------- Profit calculation settings";
extern bool      CalculateProft = true;
extern double    SpreadBuy = 1.6;
extern double    SpreadSell = 2.0;
extern bool      ExportTradeToCsv = false;

extern string    sTrading   = "------- Trading settings";
extern bool      TradeEdiEAAllowed  = false;
extern bool      AutoMagickNumber   = true;  // automaticaly generate ManualMagickNumber
extern int       ManualMagickNumber = 0;     // use this number in trades
extern bool      FilterHours        = true;  // time filtering
extern string    TradeFrom          = "0:30";
extern string    TradeTo            = "22:30";
extern bool      FilterSpread       = true;  // spread filtering - if greater no open trade
extern double    FSpreadMax         = 4.0;   // max tradable spread

// buffers
double BuyBuffer[];
double SellBuffer[];
double DiffBuffer[];
double ArrowStart[];
double ArrowStop[];

// promenne
double BBuff, SBuff, DBuff; // promenne na vymenu
double AlertArray[]; // pole s value / quantile linkami
double AlertArrayValues[]; // pole s hodnotami linek
bool AlertStackArrayB[]; // pole s aktivovanymi alerty
bool AlertStackArrayS[]; // pole s aktivovanymi alerty
bool TradeStackArrayB[]; // pole s aktivovanymi alerty
bool TradeStackArrayS[]; // pole s aktivovanymi alerty
int TradeTickArrayB[];  // tick counting
int TradeTickArrayS[];  // tick counting
int i;
double QData[]; // quantil data
int Direction = 1;
bool drawAlertLineOnlyOnce = false;
datetime newBar;   // cas posledniho baru
int qRecalcBars = 0;
int HWND = -1; // ID okna po initu
bool isInit = false; // pro zobrazovani textu
double correlValue = 0; // hodnota korelace
// profit calculation
double ProfitStackArrayBB[]; // pole s obchody
double ProfitStackArrayBS[]; // pole s obchody
double ProfitStackArraySB[]; // pole s obchody
double ProfitStackArraySS[]; // pole s obchody
double SimulatedProfit = 0.0;
bool calculatedProfit = false;
int drawLinesCount;
double StopLevelValue = 0.0;
double Spread = 0.0;
int iArraySize;

// trading EA parameter
string sClose, sOpenS, sOpenB; // nazev sledovane promenne
int iEAPeriod;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init(){
   // Name
   Indicator_Name = StringConcatenate(Indicator_Name, "_", Symbol(), DiffPair, DoubleToStr(Period(),0));
   IndicatorShortName(Indicator_Name);
   
   // Buy buffer
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,BuyBuffer);
   
   // Sell buffer
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1,SellBuffer);
   
   // Diff
   SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(2,DiffBuffer);
   
   // START
   SetIndexStyle(3,DRAW_ARROW, 0, 1);
   SetIndexArrow(3,159);
   SetIndexBuffer(3,ArrowStart);
   
   // STOP
   SetIndexStyle(4,DRAW_ARROW, 0, 1);
   SetIndexArrow(4,159);
   SetIndexBuffer(4,ArrowStop);

   // Parsing values by type
   if(AlertSystem == "Quantile"){
     valuesToDoubleArray(AlertArray, AlertQuantileArray);
   }else{
     valuesToDoubleArray(AlertArray, AlertValueArray);
   }

   // Buffer alert values
   iArraySize = ArraySize(AlertArray);
   ArrayResize(AlertStackArrayB, iArraySize);
   ArrayResize(AlertStackArrayS, iArraySize);
   ArrayResize(TradeStackArrayB, iArraySize);
   ArrayResize(TradeStackArrayS, iArraySize);
   ArrayResize(TradeTickArrayB, iArraySize);
   ArrayResize(TradeTickArrayS, iArraySize);   
   ArrayResize(AlertArrayValues, iArraySize);
   arrayInitialize(AlertStackArrayB, false);
   arrayInitialize(AlertStackArrayS, false);
   arrayInitialize(TradeStackArrayB, false);
   arrayInitialize(TradeStackArrayS, false);
   ArrayInitialize(TradeTickArrayB, 0);
   ArrayInitialize(TradeTickArrayS, 0);

   // linky vykreslovat pouze jednou
   drawAlertLineOnlyOnce = false;
   
   // nastaveni baru na prepocet
   qRecalcBars = AlertQRecalcBars;
   
   // prepocet korelace
   calculateCorrelation();
   
   // force invert
   if(ForceInvert == true){
     Direction = -1;
   }else{
     Direction = 1;
   }
   
   // nulovani profitu
   SimulatedProfit = 0.0;
   
   if(TradeEdiEAAllowed == true){
     // automaticke nastavovani MagickNumber
     if(AutoMagickNumber == true){
       ManualMagickNumber = makeMagicNumber(Symbol()+DiffPair+timeFrameToString(Period())); // vygenerujeme nove
     }
     // names of global variables sets for EA
     sClose = StringConcatenate("EDI-CLOSE_",Symbol(),"_",DiffPair,"_",ManualMagickNumber);
     sOpenB = StringConcatenate("EDI-OPENB_",Symbol(),"_",DiffPair,"_",ManualMagickNumber);
     sOpenS = StringConcatenate("EDI-OPENS_",Symbol(),"_",DiffPair,"_",ManualMagickNumber);
   }
   
   return(0);
}

// zjisteni korelace + nastaveni promennych
void calculateCorrelation(){
   // aktualni hodnota korelace, na zaklade ni se pak otaceji pary
   correlValue = getCorrelationPairs(Symbol(), DiffPair, PERIOD_D1, CorelationD1Period);
   if(CorrelationDirection == true){
     if(correlValue < 0){
       Direction = -1;
     }else{
       Direction = 1;
     }
   }
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit(){
  // delete objects
  for(int x=1;x<=13;x++) ObjectDelete(StringConcatenate(Indicator_Name,"_",x));
  // delete alert lines
  for(x=0;x<=drawLinesCount;x++) SetLevelValue(x, EMPTY_VALUE);
  return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start(){
   int limit;
   int counted_bars = IndicatorCounted(); // kolik jich uz bylo spocitano
   if(counted_bars < 0) return(-1);       // error protection
   if(counted_bars > 0) counted_bars--;   // recounted last counted bar
   limit = Bars - counted_bars;           // kolik jich mame jeste spocitat
   
   // je pozadovano pocitani pouze urciteho poctu baru (kvuli rychlosti)
   if(MaxBarCount > 0){
     if(limit > MaxBarCount){
       limit = MaxBarCount;
     }
   }

   // handle for text labels, only once   
   if(HWND < 0){
     HWND = WindowFind(Indicator_Name); // nelezeni okna indikatoru
     if(HWND < 0) return(0);
   }

   // Main cycle *****************************************************
   for(i = limit; i>=0; i--){
     // get values
     getEDiffValues(i);
     // to graph buffers
     if(ShowOnlyDiffCurve == false){
       BuyBuffer[i] = BBuff;
       SellBuffer[i] = SBuff;
     }
     DiffBuffer[i] = DBuff;
     // drawing price label
     if(i == 0) createPriceLabel(DBuff);
   }
   
   // recalculate quantile alert lines, every AlertQRecalcBars bars
   recalculateQuantilData(); 
   // draw alert lines only once
   drawAlertLines();

   // calculating spread
   if(FilterSpread == true){
     Spread = getSpreadBySymbol(Symbol()) + getSpreadBySymbol(DiffPair);
   }
   
   // resime alerty + vykreslujeme body
   calculateAlertSystem(limit);

   // modify some texts
   initText();
    
   // simulated profit calculation - only once
   calculateSimulatedProfit(MaxBarCount);
  
   return(0);
}
//+------------------------------------------------------------------+

// ziska hodnoty
void getEDiffValues(int i){
   switch(DiffType){
     case 2:
       getMAPipsType(i, MaPipsPeriod, Symbol(), DiffPair, Direction, Period(), MODE_SMA);
       break;
     case 3:
       getStochType(i, Symbol(), DiffPair, Direction, Period());
       break;
     case 1:
     default:
       getStdDevType(i, StdDevPeriod, Symbol(), DiffPair, Direction, Period(), MODE_SMA);
       break;
   }
}

// Stoch method
// function set global variable BBuff, SBuff, DBuff
// direction [1;-1]
void getStochType(int i, string BuyPair, string SellPair, int direction = 1, int TimeFrame = 0){
   // stochastic
   BBuff = smothStoch(i, TimeFrame, SmothCurveFactor, BuyPair, Kperiod, Dperiod, Slowing);
   SBuff = smothStoch(i, TimeFrame, SmothCurveFactor, SellPair, Kperiod, Dperiod, Slowing) * direction;
   DBuff = BBuff - SBuff;
}

// vyhlazeni krivky stochasticu
double smothStoch(int position, int TimeFrame, int period, string pair, int PK, int PD, int PS ){
   // vyzazeni vyhlazovani aktualniho baru
   if(NoSmothActualBar == true && position == 0){
     period = 0;
   }
   double result = 0.0;
   for(int x=0;x<=period;x++){
     result += iStochastic(pair, TimeFrame, PK, PD, PS, MODE_SMA, 0, MODE_SIGNAL, position+x);
   }
   result /= (period + 1);
   return(result);
}

// MAPips method
// function set global variable BBuff, SBuff, DBuff
// direction [1;-1]
void getMAPipsType(int i, int period, string BuyPair, string SellPair, int direction = 1, int TimeFrame = 0, int MaMethod = MODE_SMA){
   // moving average
   double MAB = iMA(BuyPair,  TimeFrame, period, 0, MaMethod, PRICE_CLOSE, i);
   double MAS = iMA(SellPair, TimeFrame, period, 0, MaMethod, PRICE_CLOSE, i);
   
   // pips distance
   MAB = convertPointPrice(BuyPair, MAB - iClose(BuyPair, TimeFrame, i));
   MAS = convertPointPrice(SellPair, MAS - iClose(SellPair, TimeFrame, i));
    
   // global variables
   BBuff = MAB;
   SBuff = MAS * direction;
   DBuff = BBuff - SBuff;
}

// StdDev method
// function set global variable BBuff, SBuff, DBuff
// direction [1;-1]
void getStdDevType(int i, int period, string BuyPair, string SellPair, int direction = 1, int TimeFrame = 0, int MaMethod = MODE_SMA){
   // moving average
   double MAB = iMA(BuyPair,  TimeFrame, period, 0, MaMethod, PRICE_CLOSE, i);
   double MAS = iMA(SellPair, TimeFrame, period, 0, MaMethod, PRICE_CLOSE, i);
      
   // standard deviation Buy
   double s = 0.0;
   for(int j = 0; j < period; j++) s += MathPow(MAB - iClose(BuyPair, TimeFrame, j+i),2);      
   double DevB = MathSqrt(s / (period-1));
   if(DevB == 0) DevB = 0.00001; // divide zero protection ;)
   
   // standard deviation Sell  
   s = 0.0;
   for(j = 0; j < period; j++) s += MathPow(MAS - iClose(SellPair, TimeFrame, j+i),2);      
   double DevS = MathSqrt(s / (period-1));
   if(DevS == 0) DevS = 0.00001; // divide zero protection ;)
      
   // global variables
   BBuff = (iClose(BuyPair, TimeFrame, i) - MAB) / DevB;
   SBuff = ((iClose(SellPair, 0, i) - MAS) / DevS) * direction;
   DBuff = BBuff - SBuff;
}

//prevede cenu na body (price1 - price2) => pips
double convertPointPrice(string symbol, double price){
 double point = MarketInfo(symbol, MODE_POINT);  
 if(point == 0.00001)
   return(price / (point*10));
 if(point == 0.0001)
   return(price / (point));
 if(point == 0.001)
   return(price / (point*10));
 if(point == 0.01)
   return(price / (point));
}

//prevede body na cenu
double convertPoint(string symbol, double price){
 double point = MarketInfo(symbol, MODE_POINT);  
 if(point == 0.00001)
   return(price * (point*10));
 if(point == 0.0001)
   return(price * (point));
 if(point == 0.001)
   return(price * (point*10));
 if(point == 0.01)
   return(price * (point));
}

// preparsovani hodnot oddelenych lomitkem do pole
void valuesToDoubleArray(double &aResult[], string &values, string delimiter = ","){
   // vymazani pole
   ArrayResize(aResult,0);
   int s = 0;
   int i = 0;
   string current;
   // dodani posledni carky kvuli podpore i jednoho 
   if (StringSubstr(values,StringLen(values),1) != delimiter){
     values = StringConcatenate(values, delimiter);
   }
   // prvni vyskyt
   i = StringFind(values, delimiter, s);
   // dokud tam neco je tak parsujeme
   while (i > 0){
     current = StringSubstr(values, s, i-s); // vyparsovani prvni hodnoty
     ArrayResize(aResult,ArraySize(aResult)+1); // pridame dalsi pozici
     aResult[ArraySize(aResult)-1] = StrToDouble(current); // pridame dalsi hodnotu
     s = i + 1;
     i = StringFind(values, delimiter, s); // dalsi vyskyt
   }
}

// vypocet hodnoty kvantilu dle vstupnich dat
// aData - double hodnoty v poli
// perceptil - hodnota v procentech (25 1.kvartil, 50 median/2. kvartil, 75 3.kvartil)
double calculateQuantile(double aData[], double perceptil = 50){
  double qKoef = perceptil * (ArraySize(aData) + 1) / 100.0; // koeficient, napr. 8.25
  // serazeni dat dle hodnot
  ArraySort(aData);
  // pokud je qKoef vetsi jak max hodnota vracime max hodnotu
  if(qKoef > ArraySize(aData)){
    return(aData[ArraySize(aData)-1]);
  }
  if(qKoef < 1){
    return(aData[0]);
  }
  // ziskani min a max hodnoty
  int qMin = MathFloor(qKoef); // 8
  int qMax = MathCeil(qKoef);  // 9
  // vytazeni vlastnich hodnot
  double dMin = aData[qMin-1]; // 10
  double dMax = aData[qMax-1]; // 20
  // linearni interpolace TODO lze interpolovat nekolika zpusoby
  double qValue = dMin + ((dMax - dMin) * (qKoef - qMin)); // 10 + ((20-10) * (8.25 - 8)) => 10 + (10*.25) = 12.5
  return(qValue);
}

// nakresli a stanovi linky
void drawAlertLines(){
   // pokud byly jiz vykresleny tak je nevykreslujeme
   if(drawAlertLineOnlyOnce == true) return(0);
   int pos = 0;
   // pokud se jedna o value tak vykreslime klasicke linky
   if(AlertSystem != "Quantile"){
     // klasicke linky dle hodnoty
     SetLevelValue(pos, 0);
     pos++;
     SetLevelStyle(STYLE_DOT, 0, DimGray);
     for(i=0; i<iArraySize; i++){
       AlertArrayValues[i] = AlertArray[i]; // zaznamename hodnoty linek
       SetLevelValue(pos, AlertArray[i]);
       pos++;
       SetLevelValue(pos, -1*AlertArray[i]);
       pos++;
     }
     StopLevelValue = AlertValueClose;
     
     createHorizontalLine(StopLevelValue, Crimson, 11);
     createHorizontalLine(StopLevelValue * (-1), Crimson, 12);
     createHorizontalLine(0, DarkOrange, 13);
     
     drawAlertLineOnlyOnce = true;
     drawLinesCount = pos-1;
   }else{
     // podle quantilu a pokud mame data
     // prevedeme data
     double qData[];
     ArrayResize(qData,MaxBarCount);
     int aSize;
     for(i = MaxBarCount; i>=0; i--){
       getEDiffValues(i);
       qData[i] = MathAbs(DBuff);
     }
     double qValue;
     SetLevelStyle(STYLE_DOT, 0, DimGray);
     for(i=0; i<iArraySize; i++){
       qValue = calculateQuantile(qData, AlertArray[i]);
       AlertArrayValues[i] = qValue; // zaznamename hodnoty linek
       SetLevelValue(pos, qValue);
       pos++;
       SetLevelValue(pos, -1*qValue);
       pos++;
     }
     if(AlertQuantileClose == 0){
       StopLevelValue = 0.0;
     }else{
       StopLevelValue = calculateQuantile(qData, AlertQuantileClose);
     }
     
     // vykreslime linky
     createHorizontalLine(StopLevelValue, Crimson, 11);
     createHorizontalLine(StopLevelValue * (-1), Crimson, 12);
     createHorizontalLine(0, DarkOrange, 13);
     
     drawAlertLineOnlyOnce = true;
     drawLinesCount = pos-1;
   }
}

// zobrazovani a vyskakovani alertu dle linek
void calculateAlertSystem(int limit){
  int iRes;
  for(i = limit; i>=0; i--){
    // ziskame data
    getEDiffValues(i);
    // reset values
    if(i != 0){
      ArrowStart[i] = EMPTY_VALUE;
      ArrowStop[i] = EMPTY_VALUE;
    }
    // projedeme alert pole
    iRes = raiseAlert(DBuff,i);
    if(iRes == -1){
      // Uzavirame!
      ArrowStop[i] = DBuff;
    }
    if(iRes > 0){
      // obchodujeme!
      ArrowStart[i] = DBuff;
    }
    // OBCHODUJEME
    iRes = tradeAlert(DBuff, i);
    if(iRes == -1){
      // uzavirame
      ArrowStop[i] = DBuff; // nova online hodnota, zaznamename hodnotu openu
      SendMessage(StringConcatenate(ShortName," ",Symbol(),"/",DiffPair," CLOSE ALL!"));
      sendEAClose();
    }
    if(iRes > 0){
      // obchodujeme!
      ArrowStart[i] = DBuff; // nova online hodnota, zaznamename hodnotu close
      sendEAOpenBuy(DBuff);
    }
  }
}

// zobrazovani a vyskakovani alertu dle linek
void calculateSimulatedProfit(int limit){
  if(CalculateProft != true) return(0);
  if(calculatedProfit == true) return(0);
  if(ExportTradeToCsv == true){
    // exportujeme do CSV souboru
    int FileHandle;
    string FileName = ShortName+"_"+Symbol()+"_"+DiffPair+"_"+Period()+"_equity_pips.csv";
    FileHandle=FileOpen(FileName,FILE_CSV|FILE_WRITE);
    FileWrite(FileHandle,"Date","PIPS","EQUITY");
  }
  int iRes;
  double dProfit;
  datetime iDate;
  for(i = limit; i>=0; i--){
    // ziskame data
    getEDiffValues(i);
    // projedeme alert pole
    iRes = raiseAlert(DBuff, -1);
    if(iRes == -1){
      dProfit = simulatedCloseTrade(i, DBuff);
      SimulatedProfit += dProfit;
      if(ExportTradeToCsv == true){
        // ukladani do souboru
        iDate = iTime(Symbol(),0,i);
        FileWrite(FileHandle,TimeToStr(iDate, TIME_DATE)+" "+TimeToStr(iDate, TIME_MINUTES), dProfit, SimulatedProfit);
      }
    }
    if(iRes > 0){
      // obchodujeme!
      simulatedOpenTrade(i, DBuff);
    }
  }
  // uzavreni souboru
  if(ExportTradeToCsv == true){
    // uzavreni
    FileClose(FileHandle);
    // send message about file
    SendMessage(StringConcatenate(ShortName," Export to CSV done! saved in experts\files\\",FileName));
  }
  string profitFrom = TimeToStr(iTime(Symbol(), 0, limit),TIME_DATE);
  TextCreate(StringConcatenate("SimulatedProfit: ", DoubleToStr(SimulatedProfit,1)," pips from: ",profitFrom), 4, Gray, 4, 48);
  calculatedProfit = true;
}

// simulovane otevreni obchodu
void simulatedOpenTrade(int position, double dBuffValue){
  double dValue = 0.0;
  if(dBuffValue > 0){
    // BuyPair sell; SellPair buy
    if(Direction == 1){
      // sell za Bid
      addSimulatedTrade(ProfitStackArrayBS, iClose(Symbol(), 0, position));
      // buy za Ask
      addSimulatedTrade(ProfitStackArraySB, iClose(DiffPair, 0, position) + convertPoint(DiffPair, SpreadSell));
    }else{
      // sell za Bid
      addSimulatedTrade(ProfitStackArrayBS, iClose(Symbol(), 0, position));
      // sell za Bid
      addSimulatedTrade(ProfitStackArraySS, iClose(DiffPair, 0, position));
    }
  }else{
    // BuyPair buy; SellPair sell
    if(Direction == 1){
      // buy za Ask
      addSimulatedTrade(ProfitStackArrayBB, iClose(Symbol(), 0, position) + convertPoint(Symbol(), SpreadBuy));
      // sell za Bid
      addSimulatedTrade(ProfitStackArraySS, iClose(DiffPair, 0, position));
    }else{
      // buy za Ask
      addSimulatedTrade(ProfitStackArrayBB, iClose(Symbol(), 0, position) + convertPoint(Symbol(), SpreadBuy));
      // buy za Ask
      addSimulatedTrade(ProfitStackArraySB, iClose(DiffPair, 0, position) + convertPoint(DiffPair, SpreadSell));
    }
  }
  
}

// pridani zaznamu do pole
void addSimulatedTrade(double &ProfitStackArray[], double price){
  ArrayResize(ProfitStackArray, ArraySize(ProfitStackArray)+1);
  ProfitStackArray[ArraySize(ProfitStackArray)-1] = price;
}

// simulovane uzavreni obchodu - vraci pocet pips
double simulatedCloseTrade(int position, double dBuffValue){
  // uzavirani a pocitani pozic
  double dValue = 0.0;
  int j;
  // buy buy
  for(j=0; j<ArraySize(ProfitStackArrayBB); j++){
    dValue += convertPointPrice(Symbol(), iClose(Symbol(), 0, position) - ProfitStackArrayBB[j]);
  }
  // sell buy
  for(j=0; j<ArraySize(ProfitStackArraySB); j++){
    dValue += convertPointPrice(DiffPair, iClose(DiffPair, 0, position) - ProfitStackArraySB[j]);
  }
  // buy sell
  for(j=0; j<ArraySize(ProfitStackArrayBS); j++){
    dValue += convertPointPrice(Symbol(),ProfitStackArrayBS[j] - (iClose(Symbol(), 0, position) + convertPoint(Symbol(), SpreadBuy)));
  }
  // sell sell
  for(j=0; j<ArraySize(ProfitStackArraySS); j++){
    dValue += convertPointPrice(DiffPair, ProfitStackArraySS[j] - (iClose(DiffPair, 0, position) + convertPoint(DiffPair, SpreadSell)));
  }
  // reset promennych
  ArrayResize(ProfitStackArrayBB,0);
  ArrayResize(ProfitStackArrayBS,0);
  ArrayResize(ProfitStackArraySB,0);
  ArrayResize(ProfitStackArraySS,0);
  // vracime profit
  return(dValue);
}

// mame vyvolat alert?
// vraci 0 - nic se nedeje; N > 0 - alert cislo; -1 - close all
// dValue - aktualni hodnota (diff value)
int raiseAlert(double dValue, int i){
  if(i == 0) return(0);
  // projedeme pole alertu
  int iRet = 0;
  for(int j=0; j<ArraySize(AlertArrayValues); j++){
    // pro sell
    if(AlertStackArrayS[j] == false){
      // pokud jsme toto jeste preprojeli
      if(dValue > AlertArrayValues[j]){
        // mame prekrocenou hodnotu Alert!
        AlertStackArrayS[j] = true;
        // vracime znacku
        iRet = j+1;
      }
    }else{
      if(dValue <= (StopLevelValue * (-1))){
        // resetujeme promenne
        arrayInitialize(AlertStackArrayS, false);
        // vracime znacku
        return(-1);
      }
    }
    // pro buy 
    if(AlertStackArrayB[j] == false){
      // pokud jdme toto jeste preprojeli
      if(dValue < (AlertArrayValues[j] * (-1))){
        // mame prekrocenou hodnotu Alert!
        AlertStackArrayB[j] = true;
        // vracime znacku
        iRet = j+1;
      }
    }else{
      if(dValue >= StopLevelValue){
        // resetujeme promenne
        arrayInitialize(AlertStackArrayB, false);
        // TODO zavirame
        // vracime znacku
        return(-1);
      }
    }
  }
  return(iRet);
}

// mame vyvolat trade alert?
// trade alert funguje pouze pokud je aktualni bar 0
// vraci 0 - nic se nedeje; N > 0 - alert cislo; -1 - close all
// dValue - aktualni hodnota (diff value)
int tradeAlert(double dValue, int i){
  if(i != 0) return(0);
  // projedeme pole alertu
  int iRet = 0;
  for(int j=0; j<ArraySize(AlertArrayValues); j++){
    // pro sell
    if(TradeStackArrayS[j] == false){
      // pokud jsme toto jeste neprojeli
      if(dValue > AlertArrayValues[j]){
        // zvysime tick u tohoto pole a dokud nebude MinTickCount tak alert nevyvolavame
        TradeTickArrayS[j]++;
        // pokud jiz mame tak vyvolavame Alert
        if(TradeTickArrayS[j] >= AlertMinTickCount){
          // mame prekrocenou hodnotu Alert!
          TradeStackArrayS[j] = true;
          // vracime znacku
          iRet = j+1;
          // reset tickcount
          TradeTickArrayS[j] = 0;
        }
      }else{
        TradeTickArrayS[j] = 0; // reset tickcount
      }
    }else{
      if(dValue <= (StopLevelValue * (-1))){
        // resetujeme promenne
        arrayInitialize(TradeStackArrayS, false);
        ArrayInitialize(TradeTickArrayS, 0);
        // vracime znacku na zavreni
        return(-1);
      }
    }
    // pro buy 
    if(TradeStackArrayB[j] == false){
      // pokud jdme toto jeste preprojeli
      if(dValue < (AlertArrayValues[j] * (-1))){
        // zvysime tick u tohoto pole a dokud nebude MinTickCount tak alert nevyvolavame
        TradeTickArrayB[j]++;
        // pokud jiz mame tak vyvolavame Alert
        if(TradeTickArrayB[j] >= AlertMinTickCount){
          // mame prekrocenou hodnotu Alert!
          TradeStackArrayB[j] = true;
          // vracime znacku na otevreni
          iRet = j+1;
          // reset tickcount
          TradeTickArrayB[j] = 0;
        }
      }
    }else{
      if(dValue >= StopLevelValue){
        // resetujeme promenne
        arrayInitialize(TradeStackArrayB, false);
        ArrayInitialize(TradeTickArrayB, 0);
        // vracime znacku
        return(-1);
      }
    }
  }
  return(iRet);
}

// prepocet quantil dat
void recalculateQuantilData(){
  if(AlertSystem != "Quantile") return(0); // nyni nic neresime
  if(isNewBar() == true){
    // mameno novy bar
    qRecalcBars--;
    if(qRecalcBars <= 0){
      // provadime prepocet
      qRecalcBars = AlertQRecalcBars;
      drawAlertLineOnlyOnce = false;
    }
  }
}

bool isNewBar(){
  // obchodujeme pouze na novy bar
  if(newBar != iTime(NULL, Period(), 1)){
    newBar = iTime(NULL, Period(), 1);
    return(true);
  }else{
    return(false); // neobchodujeme
  }
}

// odesilani zprav
void SendMessage(string message, bool toComment = false){
  Print(message);
  if(SendAlert == true) Alert(message);
  if(SendEmail == true) SendMail(StringConcatenate(TimeToStr(TimeCurrent(), TIME_SECONDS)," ",message),"");
  if(toComment == true) Comment(message);
}

// zobrazi texty
void initText(){

   // zobrazovat aktualni stav levelu
   if(ShowAlertLevelStatus == true){
     string sLevelsB = "";
     string sLevelsS = "";
     for(int j=0; j<ArraySize(AlertArrayValues); j++){
       if(TradeStackArrayB[j] == false){
         // jeste nebyl aktivovan
         sLevelsB = StringConcatenate(sLevelsB, " [", TradeTickArrayB[j], "]");
       }else{
         // jiz byl aktivovan
         sLevelsB = StringConcatenate(sLevelsB, " [X]");
       }
       if(TradeStackArrayS[j] == false){
         // jeste nebyl aktivovan
         sLevelsS = StringConcatenate(sLevelsS, " [", TradeTickArrayS[j], "]");
       }else{
         // jiz byl aktivovan
         sLevelsS = StringConcatenate(sLevelsS, " [X]");
       }
     }
     // vypsani textu
     TextCreate("BLevels: "+sLevelsB, 5, CornflowerBlue, 4, 70);
     TextCreate("SLevels: "+sLevelsS, 6, CornflowerBlue, 4, 59);
   }
   
   // zobrazovani spreadu
   color Clr = SeaGreen;
   if(FilterSpread == true){
     if(Spread > FSpreadMax) Clr = Red;
     TextCreate("Spread: "+DoubleToStr(Spread, 2), 10, Clr, 4, 103);
   }
   
   // zobrazovani filtrace casu
   if(FilterHours == true){
     if(isTradeHours(TradeFrom, TradeTo) == true){
       Clr = SeaGreen;
     }else{
       Clr = Red;
     }
     TextCreate("TradeTime: "+TradeFrom+" - "+TradeTo, 8, Clr, 4, 92);
   }
   
   if(isInit == true) return(true);
   
   // korelace
   TextCreate(StringConcatenate("Corelation: ", DoubleToStr(correlValue*100,0),"% / ",CorelationD1Period,"D1"), 3, Gold, 4, 37);
   
   // zobrazovani magickNumbers
   if(ShowMagickNumber == true){
     TextCreate("MagickNumber: "+ManualMagickNumber, 7, Peru, 4, 81);
   }
   
   if(Direction == 1){
     TextCreate(StringConcatenate("BUY:", Symbol()),  1, ForestGreen, 4, 26);
     TextCreate(StringConcatenate("SELL:", DiffPair), 2, Tomato,      4, 15);
   }else{
     TextCreate(StringConcatenate("BUY:",Symbol(),",",DiffPair),  1, ForestGreen, 4, 26);
     TextCreate("SELL:", 2, Tomato,      4, 15);
   }
   
   isInit = true; // provedeme pouze jednou
   return(true);
}

// vypisuje textove hlaseni do okna indikatoru
void TextCreate(string txt, int ObjId = 0, color col = White, int x = 0, int y = 0, int FontSize = 7){
   if(HWND < 0) return(0);
   string ID = StringConcatenate(Indicator_Name, "_", ObjId);
   if(ObjectFind(ID) != 1){
     ObjectCreate(ID, OBJ_LABEL, HWND, 0, 0);
     ObjectSet(ID, OBJPROP_XDISTANCE, x);
     ObjectSet(ID, OBJPROP_YDISTANCE, y);
   }
   ObjectSetText(ID, txt, FontSize, "Verdana", col);
}

// vypocet korelace
double getCorrelationPairs(string Symbol1, string Symbol2, int TimeFrame, int periods, int position = 0){
	datetime closeTime1	= iTime(Symbol1,TimeFrame,position),
				closeTime2	= iTime(Symbol2,TimeFrame,position),
				closeTime	= MathMin(closeTime1,closeTime2);
	int		shift1		= iBarShift(Symbol1,TimeFrame,closeTime),
				shift2		= iBarShift(Symbol2,TimeFrame,closeTime);
	double	Co,
	         close1[],
				close2[];
	ArrayCopySeries(close1,MODE_CLOSE,Symbol1,TimeFrame);
	ArrayCopySeries(close2,MODE_CLOSE,Symbol2,TimeFrame);
	int bars = MathMin(ArraySize(close1)-shift1,ArraySize(close2)-shift2);
	if ( periods > 0 )
	     bars = periods;
   Co = Correlation(close1,close2,shift1,shift2,bars);
   return(Co);
}

//+------------------------------------------------------------------+
//| Correlation Coefficient R														|
//+------------------------------------------------------------------+
double Correlation(double x[], double y[], int x_shift = 0, int y_shift = 0, int count = -1){
	int n = MathMin(ArraySize(x)-x_shift,ArraySize(y)-y_shift);
	if(n>count && count>0)
		n=count;
	if(n<2)
		return(-2);
	double	sum_sq_x,
				sum_sq_y,
				sum_coproduct,
				mean_x = x[x_shift],
				mean_y = y[y_shift];
	for(int i = 0; i < n; i++){
		double	sweep = i / (i+1.0),
					delta_x = x[i+x_shift] - mean_x,
					delta_y = y[i+y_shift] - mean_y;
		sum_sq_x += delta_x*delta_x * sweep;
		sum_sq_y += delta_y*delta_y * sweep;
    	sum_coproduct += delta_x*delta_y * sweep;
    	mean_x += delta_x / (i+1.0);
    	mean_y += delta_y / (i+1.0);
	}
	double	pop_sd_x = MathSqrt(sum_sq_x/n),
				pop_sd_y = MathSqrt(sum_sq_y/n),
				cov_x_y = sum_coproduct / n;
	if(pop_sd_x*pop_sd_y != 0.0)
		return(cov_x_y / (pop_sd_x*pop_sd_y));
	return(-3);
}

/**
* create a positive integer for the use as a magic number.
*
* The function takes a string as argument and calculates
* an 31 bit hash value from it. The hash does certainly not 
* have the strength of a real cryptographic hash function 
* but it should be more than sufficient for generating a
* unique ID from a string and collissions should not occur.
*
* use it in your init() function like this: 
*    magic = makeMagicNumber(WindowExpertName() + Symbol() + Period());
*
* where name would be the name of your EA. Your EA will then
* get a unique magic number for each instrument and timeframe
* and this number will always be the same, whenever you put
* the same EA onto the same chart.
*
* Numbers generated during testing mode will differ from those
* numbers generated on a live chart.
*/
int makeMagicNumber(string key){
   int i, k;
   int h = 0;
   if (IsTesting()){
      key = "_" + key;
   }
   for (i=0; i<StringLen(key); i++){
      k = StringGetChar(key, i);
      h = h + k;
      h = bitRotate(h, 5); // rotate 5 bits
   }
   for (i=0; i<StringLen(key); i++){
      k = StringGetChar(key, i);
      h = h + k;
      // rotate depending on character value
      h = bitRotate(h, k & 0x0000000F);
   }
   // now we go backwards in our string
   for (i=StringLen(key); i>0; i--){   
      k = StringGetChar(key, i - 1);
      h = h + k;
      // rotate depending on the last 4 bits of h
      h = bitRotate(h, h & 0x0000000F); 
   }
   return(h & 0x7fffffff);
}

/**
* Rotate a 32 bit integer value bit-wise 
* the specified number of bits to the right.
* This function is needed for calculations
* in the hash function makeMacicNumber()
*/
int bitRotate(int value, int count){
   int i, tmp, mask;
   mask = (0x00000001 << count) - 1;
   tmp = value & mask;
   value = value >> count;
   value = value | (tmp << (32 - count));
   return(value);
}

// prevody timeframe
string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

// vraci string timeframe podle hodnoty
string timeFrameToString(int tf){
   for (int i=ArraySize(iTfTable)-1; i>=0; i--){
     if (tf==iTfTable[i]){
       return(sTfTable[i]);
     }
   }
   return("");
}

// odeslani uzavreni do EAcka
void sendEAClose(){
  if(TradeEdiEAAllowed != true) return;
  GlobalVariableSet(sClose, 1);
  sendEATick();
}

// otevreni obchodu v EA
void sendEAOpenBuy(double &DBuff){
  if(TradeEdiEAAllowed != true) return;
  // filtrace na cas
  if(FilterHours == true && isTradeHours(TradeFrom, TradeTo) != true) return;
  // filtrace spreadu
  if(FilterSpread == true && Spread > FSpreadMax) return;
  // smer obchodu
  if(DBuff > 0){
    // sell
    GlobalVariableSet(sOpenS, 1);
  }else{
    // buy
    GlobalVariableSet(sOpenB, 1);
  }
  SendMessage(StringConcatenate(ShortName," ",Symbol(),"/",DiffPair," TRADE!"));
  sendEATick();
}

#import "user32.dll"
   int PostMessageA(int hWnd, int Msg, int wParam, int lParam);
   int RegisterWindowMessageA(string lpString);
#import

// odeslani ticku EAck
void sendEATick(){
  int hWnd = WindowHandle(Symbol(), Period());
  if(hWnd == 0) return(0);
  int MT4InternalMsg = RegisterWindowMessageA("MetaTrader4_Internal_Message");
  PostMessageA(hWnd, MT4InternalMsg, 2, 1); // fake tick send
}

// Test zda jsme v danem case, POZOR v testu vraci true!
bool isTradeHours(string dFrom = "23:00", string dTo = "23:30"){
  if(IsTesting() == true) return(true); // v prubehu restovani 
  // prepocet casu na desetine cislo
  datetime FromHour = StrToTime(dFrom);
  datetime ToHour = StrToTime(dTo);
  datetime Now = StrToTime(StringConcatenate(Hour(),":",Minute()));
  if(FromHour < ToHour){
    // klasicke reseni
    if(FromHour <= Now && Now <= ToHour){
      return(true);
    }else{
      return(false);
    }
  }else{
    // je to obracene tedy musime do pulnoci a od pulnoci dal
    datetime MidnightB = StrToTime("23:59:59");
    datetime MidnightA = StrToTime("00:00:00");
    if((FromHour <= Now && Now <= MidnightB) || (Now >= MidnightA && Now <= ToHour)){
      return(true);
    }else{
      return(false);
    }
  }
}

// vytvori ukazatel ceny pokud o nej mame zajem
void createPriceLabel(double price){
   if(ShowPriceLabel != true) return;
   if(HWND < 0) return(0);
   string ID = StringConcatenate(Indicator_Name,"_9");
   if(ObjectFind(ID) != 1){
     ObjectCreate(ID, OBJ_ARROW, HWND, Time[0], price);
     ObjectSet(ID, OBJPROP_COLOR, DodgerBlue);
     ObjectSet(ID, OBJPROP_ARROWCODE, 6);
   }
   ObjectSet(ID, OBJPROP_PRICE1, price);
   ObjectSet(ID, OBJPROP_TIME1, Time[0]);
}

// velikost spreadu v bodech
double getSpreadBySymbol(string symbol){ 
 double point = MarketInfo(symbol, MODE_POINT);  
 if(point == 0.00001)
   return((MarketInfo(symbol, MODE_ASK) - MarketInfo(symbol, MODE_BID)) / (point*10));
 if(point == 0.0001)
   return((MarketInfo(symbol, MODE_ASK) - MarketInfo(symbol, MODE_BID)) / (point));
 if(point == 0.001)
   return((MarketInfo(symbol, MODE_ASK) - MarketInfo(symbol, MODE_BID)) / (point*10));
 if(point == 0.01)
   return((MarketInfo(symbol, MODE_ASK) - MarketInfo(symbol, MODE_BID)) / (point));
}

// vytvori linku
void createHorizontalLine(double value, color Clr, int number){
   if(HWND < 0) return(0);
   string ID = StringConcatenate(Indicator_Name,"_",number);
   ObjectCreate(ID, OBJ_HLINE, HWND, 0, value);
   ObjectSet(ID, OBJPROP_COLOR, Clr);
   ObjectSet(ID, OBJPROP_STYLE, STYLE_DOT);
   ObjectSet(ID, OBJPROP_WIDTH, 0);
   ObjectSet(ID, OBJPROP_BACK, true);
}

// premazavani promennych
void arrayInitialize(bool &array[], bool value){
  for(int j=0; j<ArraySize(array); j++){
    array[j] = value;
  }
}